#!/bin/bash
# Aufräumen eines Linux-Systems mit APT und GRUB
# v2.05
# Author: Michael Strusch
# Last Modification: 27.04.2024
# Purpose: Diese Shell entfernt alte Kernel, nicht mehr notwendige Pakete und aktualisiert das System.

# Dateipfad für temporäre Benutzerdatei
usr_datei=".usr_datei"

# Benutzer und Hostinformationen
akt_usr=$(whoami)
pc=$(hostname)

# Funktion zum Löschen der temporären Benutzerdatei
cleanup() {
    rm -f "$usr_datei"
}

# Prüfen und Speichern des Benutzerkennworts
echo "$akt_usr" > "$usr_datei"
clear
echo "Das Kennwort des Benutzers $akt_usr ist erforderlich!"
sudo echo "root" > "$usr_datei"
read -r val_usr < "$usr_datei"

if [ "$val_usr" = "root" ]; then
    clear
    echo "Danke! :)"
    sleep 1
    clear
    echo "Identität wurde bestätigt. Willkommen $akt_usr!"
    sleep 1
    clear
    echo "Aufräumen von $pc..."
    sleep 1
    clear
    
    # Aufräumarbeiten abhängig von verfügbaren Paketmanagern
    if command -v apt &> /dev/null; then
        # Bereinigung von Systemdateien
         echo "Nicht mehr benötigte Installationspakete werden entfernt!"
        sudo apt autoclean -y
        sudo apt autoremove --purge -y
        sudo apt autopurge -y
        sleep 1
        clear
    else
        echo "APT ist nicht installiert."
        sleep 1
    fi
    clear

    # Reparatur beschädigter Pakete und Aktualisierungen
    echo "Beschädigte Pakete werden repariert!"
    sleep 1
    if command -v dpkg &> /dev/null; then
        sudo dpkg --configure -a
    else
        echo "DPKG ist nicht installiert."
        sleep 1
    fi
    if command -v apt &> /dev/null; then
        sudo apt install --fix-broken -y
    else
        echo "APT ist nicht installiert."
        sleep 1
    fi
    sleep 1
    clear

    # Aktualisierungen und Bereinigungen
    echo "Aktualisierungen werden durchgeführt!"
    sleep 1
    if command -v flatpak &> /dev/null; then
        sudo flatpak update
    else
        echo "Flatpak ist nicht installiert."
        sleep 1
    fi
    if command -v snap &> /dev/null; then
        sudo snap refresh
    else
        echo "Snap ist nicht installiert."
        sleep 1
    fi
    if command -v apt &> /dev/null; then
        sudo apt update -y --fix-missing
    else
        echo "APT ist nicht installiert."
        sleep 1
    fi
    sleep 1
    clear

    # Kernelverwaltung
    echo "Entfernen nicht aktiver Kernel."
    sleep 1
    if command -v dpkg &> /dev/null; then
        sudo dpkg -l 'linux-*' | awk '/^ii/{print $2}' | grep -v "$(uname -r)" | grep -E 'image|headers' | xargs sudo apt-get -y purge
    else
        echo "DPKG ist nicht installiert."
        sleep 1
    fi
    echo "Starte Kernel-Update."
    sleep 1
    if command -v apt &> /dev/null; then
        sudo apt install -y linux-generic
    else
        echo "APT ist nicht installiert."
        sleep 1
    fi
    latest_kernel=$(dpkg --list | grep 'linux-image' | awk '{ print $2 }' | grep -v $(uname -r) | sort -V | tail -n 1)
    echo "Aktiviere den neuesten Kernel: $latest_kernel"
    sleep 1
    sudo update-initramfs -u
    sudo update-grub
    sudo grub-set-default "$latest_kernel"
    echo "Kernel-Update abgeschlossen."
    sleep 1
    clear

    # Vollständiges Distributions-Update
    echo "Distributions-Update wird durchgeführt."
    sleep 2
    if command -v apt &> /dev/null; then
        sudo apt upgrade -y --ignore-hold --allow-change-held-packages
        sudo apt full-upgrade -y
        sudo apt dist-upgrade -y
    else
        echo "APT ist nicht installiert."
        sleep 1
    fi
    clear
    
    #Cleanup
    echo "Alte ALog-Dateien und temporäre Dateien werden falls möglich gelöscht!"
    sleep 1
    sudo find /var/log -mtime +1 -a -type f -print0 | xargs -0 sudo rm -f
    cleanup
    sleep 1
    clear

    # Abschluss und Neustart
    echo "Das war es schon, Aufräumen abgeschlossen. Ich habe fertig. :)"
    sleep 1
    clear
    echo "Dieses Fenster darf ab jetzt wieder geschlossen werden, sonst folgt ein System-Neustart!"
    sleep 3
    clear
    echo "Okay, System-Neustart erfolgt in T-Minus: 10!"
    sleep 1
    clear
    for i in {9..1}; do
        echo "T-Minus: $i!"
        sleep 1
        clear
    done
    echo "Bis bald $akt_usr!"
    sleep 1
    clear
    sudo reboot now
else
    # Wenn kein Root-Zugriff vorhanden ist
    cleanup
    clear
    echo "Tut mir leid, keine ausreichenden Systemrechte, Skriptlauf wird abgebrochen! :("
    sleep 6
fi